<?php

declare(strict_types=1);

namespace Drupal\livewire_field\Plugin\Field\FieldType;

use Drupal\Core\Field\Attribute\FieldType;
use Drupal\Core\Field\FieldItemBase;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\Core\TypedData\DataDefinition;

/**
 * Defines the 'wire_component' field type.
 */
#[FieldType(
  id: 'wire_component',
  label: new TranslatableMarkup('Wire Component'),
  description: new TranslatableMarkup('Stores an instance of livewire Component.'),
  default_widget: 'wire_component_widget',
  default_formatter: 'wire_component_formatter',
)]
final class WireComponentFieldType extends FieldItemBase {

  /**
   * {@inheritdoc}
   */
  public function isEmpty(): bool {
    return match ($this->get('plugin_id')->getValue()) {
      NULL, '' => TRUE,
      default => FALSE,
    };
  }

  /**
   * {@inheritdoc}
   */
  public static function propertyDefinitions(FieldStorageDefinitionInterface $field_definition): array {
    // @DCG
    // See /core/lib/Drupal/Core/TypedData/Plugin/DataType directory for
    // available data types.
    $properties['plugin_id'] = DataDefinition::create('string')
      ->setLabel(t('Plugin ID'))
      ->setRequired(TRUE);

    return $properties;
  }

  /**
   * {@inheritdoc}
   */
  public static function schema(FieldStorageDefinitionInterface $field_definition): array {
    $columns = [
      'plugin_id' => [
        'type' => 'varchar',
        'not null' => FALSE,
        'length' => 255,
      ],
    ];

    $schema = [
      'columns' => $columns,
    ];

    return $schema;
  }

  /**
   * @return array|array[]
   */
  public static function defaultFieldSettings() {
    return [
        'allowed_plugins' => [],
      ] + parent::defaultFieldSettings();
  }

  /**
   * @param array $form
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *
   * @return array
   */
  public function fieldSettingsForm(array $form, FormStateInterface $form_state) {
    $elements = parent::fieldSettingsForm($form, $form_state);

    $plugin_manager = \Drupal::service('plugin.manager.wire');
    $options = [];
    foreach ($plugin_manager->getDefinitions() as $id => $definition) {
      if ($id === "broken") {
        continue;
      }
      $options[$id] = $definition['label'] ?? $id;
    }

    $elements['allowed_plugins'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Allowed Wire Components'),
      '#options' => $options,
      '#default_value' => $this->getSetting('allowed_plugins'),
      '#description' => $this->t('Select which components should be available for editors.'),
    ];

    return $elements;
  }

}
