<?php

declare(strict_types=1);

namespace Drupal\livewire_field\Plugin\Field\FieldWidget;

use Drupal\Core\Field\Attribute\FieldWidget;
use Drupal\Core\Field\FieldDefinitionInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\Field\WidgetBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\StringTranslation\TranslatableMarkup;
use Drupal\wire\Plugin\WirePluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Defines the 'wire_component_widget' field widget.
 */
#[FieldWidget(
  id: 'wire_component_widget',
  label: new TranslatableMarkup('Wire Component Selector'),
  field_types: ['wire_component'],
)]
final class WireComponentWidget extends WidgetBase {

  /**
   * @var \Drupal\wire\Plugin\WirePluginManager
   */
  protected WirePluginManager $wireComponentManager;

  public function __construct($plugin_id, $plugin_definition, FieldDefinitionInterface $field_definition, array $settings, array $third_party_settings, WirePluginManager $wire_plugin_manager) {
    parent::__construct($plugin_id, $plugin_definition, $field_definition, $settings, $third_party_settings);
    $this->wireComponentManager = $wire_plugin_manager;
  }

  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $plugin_id,
      $plugin_definition,
      $configuration['field_definition'],
      $configuration['settings'],
      $configuration['third_party_settings'],
      $container->get('plugin.manager.wire')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function formElement(FieldItemListInterface $items, $delta, array $element, array &$form, FormStateInterface $form_state): array {
    $field_settings = $this->fieldDefinition->getSettings();
    $allowed_plugins = array_filter($field_settings['allowed_plugins'] ?? []);

    if (empty($allowed_plugins)) {
      foreach ($this->wireComponentManager->getDefinitions() as $id => $definition) {
        if ($id === 'broken') {
          continue;
        }
        $allowed_plugins[$id] = $definition['label'] ?? $id;
      }
    }

    $element['plugin_id'] = [
      '#type' => 'select',
      '#title' => $this->t('Wire Component'),
      '#options' => $allowed_plugins,
      '#empty_option' => $this->t('- None -'),
      '#default_value' => $items[$delta]->plugin_id ?? '',
    ];
    return $element;
  }

}
